import React from "react";
import PropTypes from "prop-types";
import styling from "@devcake/styling";
import colors from "./colors";
import { Icon } from "../Icon";
import { Color, ColorPropType, Type, TypePropType } from "../types";
import { useStyle } from "../Scope";

styling.setDefault("components.button.icons.class", "w-5 h-5 -ml-1 mr-3 animate-spin");
styling.link("components.button.icons.loading", "components.icons.loading");

export interface ButtonProps {
  children: React.ReactNode;
  className?: string;
  disabled?: boolean;
  loading?: boolean;
  color?: Color;
  onClick: () => void;
  type: Type;
}

export const Button = ({
  children,
  className,
  color,
  onClick,
  type,
  disabled,
  loading,
}: ButtonProps) => {
  const [loadIcon, loadClass] = useStyle(["loading", "class"], "components.button.icons");
  const [containerStyle, disabledStyle, typeStyle] = useStyle(["", "disabled", type], "components.button");
  return (
    <button
      disabled={disabled}
      className={[
        className,
        containerStyle,
        disabled ? disabledStyle : "",
        typeStyle,
        color && !disabled ? colors[color][type] : ""
      ].join(" ")}
      onClick={onClick}
    >
      {loading && (
        <Icon icon={loadIcon} className={loadClass} />
      )}
      {children}
    </button>
  );
};

Button.propTypes = {
  loading: PropTypes.bool,
  disabled: PropTypes.bool,
  children: PropTypes.node.isRequired,
  className: PropTypes.string,
  color: ColorPropType,
  onClick: PropTypes.func.isRequired,
  type: TypePropType.isRequired
};
Button.defaultProps = {
  onClick: () => null,
  type: "primary",
  disabled: false,
};
