import React from "react";
import PropTypes from "prop-types";
import styling from "@devcake/styling";
import { useStyle } from "../Scope";

styling.setDefault("components.card.container", "bg-white p-6 rounded-lg shadow-lg");
styling.setDefault("components.card.title", "text-2xl font-bold mb-Z text-gray-800");
styling.setDefault("components.card.description", "text-gray-700");

export interface CardProps {
  title: string;
  className?: string;
  description?: string;
  children: JSX.Element;
}

export const Card = ({
  title,
  children,
  className,
  description,
}: CardProps) => {
  const [containerStyle, titleStyle, descriptionStyle] = useStyle(["container", "title", "description"], "components.card");
  return (
    <div className={[containerStyle, className].join(" ")}>
      <h2 className={titleStyle}>
        {title}
      </h2>
      <p className={descriptionStyle}>{description}</p>
      {children}
    </div>
  );
};

Card.propType = {
  children: PropTypes.node,
  title: PropTypes.string.isRequired,
  description: PropTypes.string.isRequired,
};
